﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CapstoneProject.EngineClasses
{
    class Engine
    {
        // attributes
        World world;
        int worldLength = 9;
        int worldHeight = 9;

        int timestep = 0; // a timestep counter to keep track of the current timestep

        FileManager fileManager;

        public int Timestep
        {
            get { return timestep;}
        }

        //Constructors
        /// <summary>
        /// default constructor, creates a 9x9 world
        /// </summary>
        public Engine()
        {
            worldLength = 9;
            worldHeight = 9;
            world = new World(worldLength, worldHeight);

            fileManager = new FileManager();
        }

        /// <summary>
        /// Custom world size constructor. Creates a world with a custom world size
        /// </summary>
        /// <param name="diffLength"> the custom length value. Cannot be less than 1 or greater than 21</param>
        /// <param name="diffHeight"> the custom height value. Cannot be less than 1 or greater than 21</param>
        public Engine(int diffLength, int diffHeight)
        {
            // check input values against limitations
            if(diffLength >= 1 && diffLength<=21)
                worldLength = diffLength;
            else
                worldLength = 9;
            if (diffHeight >= 1 && diffHeight <= 21)
                worldHeight = diffHeight;
            else
                worldHeight = 9;
            // create a new world with custom size parameters
            world = new World(worldLength, worldHeight);

            fileManager = new FileManager();
        }

        // Mutators
        /// <summary>
        /// Tells the engines current world class to run the next timestep and update the world
        /// </summary>
        public void runTimeStep()
        {
            world.runTimeStep();
            timestep++; // increment current timestep
            fileManager.WriteWorldToFiles(returnCellGrid(), world.Bands, timestep, world.TotalBirths, world.TotalDeaths);
        }

        /// <summary>
        /// Recreates the engines world using the default values for length and height
        /// </summary>
        public void createNewWorld()
        {
            worldLength = 9;
            worldHeight = 9;
            world = new World(worldLength, worldHeight);
            timestep = 0; // reset timestep counter
            // a new world requires a reset of data so a new file manager is created
            fileManager.closeWriterStreams();
            fileManager = new FileManager();
        }

        /// <summary>
        /// Recreates the engines world using custom length and height values
        /// </summary>
        /// <param name="diffLength"> the custom length value. Cannot be less than 1 or greater than 21</param>
        /// <param name="diffHeight"> the custom height value. Cannot be less than 1 or greater than 21</param>
        public void createNewWorld(int diffLength, int diffHeight)
        {
            // check input values against limitations
            if (diffLength >= 1 && diffLength <= 21)
                worldLength = diffLength;
            else
                worldLength = 9;
            if (diffHeight >= 1 && diffHeight <= 21)
                worldHeight = diffHeight;
            else
                worldHeight = 9;
            // create a new world with custom size parameters
            world = new World(worldLength, worldHeight);
            timestep = 0; // reset timestep counter
            // a new world requires a reset of data so a new file manager is created
            fileManager.closeWriterStreams(); // close streams before creating new object to ensure streams are closed before we try to open new ones to the files
            fileManager = new FileManager();
        }

        public void createNewWorld(int diffLength, int diffHeight, int initialBands, int startingMembers)
        {
            // check input values against limitations
            if (diffLength >= 1 && diffLength <= 21)
                worldLength = diffLength;
            else
                worldLength = 9;
            if (diffHeight >= 1 && diffHeight <= 21)
                worldHeight = diffHeight;
            else
                worldHeight = 9;
            // create a new world with custom size parameters
            world = new World(worldLength, worldHeight, initialBands, startingMembers);
            timestep = 0; // reset timestep counter
            // a new world requires a reset of data so a new file manager is created
            fileManager.closeWriterStreams(); // close streams before creating new object to ensure streams are closed before we try to open new ones to the files
            fileManager = new FileManager();
        }

        public void worldData(ref int timestep, ref List<float> ATPValues, ref List<float> ATBValues, ref List<float> ATFValues, ref List<float> TPValues, ref List<float> TBValues, ref List<float> TDValues, ref List<float> BBTValues, ref List<float> DBTValues)
        {
            timestep = this.timestep;
            fileManager.ReadFromFile(ref ATPValues, ref ATBValues, ref ATFValues, ref TPValues, ref TBValues, ref TDValues, ref BBTValues, ref DBTValues);
        }

        /// <summary>
        /// Returns the cell grid from the world class
        /// </summary>
        /// <returns></returns>
        public Cell[,] returnCellGrid()
        {
            return world.Cells;
        }
    }
}
