﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using CapstoneProject.EngineClasses;

namespace CapstoneProject.GUI
{
    // The form container is a simple form that allows transition between as well as facilitates communication of data from one screen to another
    public partial class FormContainer : Form
    {
        GridGUIForm gridForm; // the grid form
        StatisticsForm statForm; // the statistics form
        NewWorldForm newWorldForm; // the new world form

        Form currentForm; // the form currently being displayed

        public FormContainer()
        {
            InitializeComponent();
            this.IsMdiContainer = true;
            // initialize the forms
            gridForm = new GridGUIForm(this);
            gridForm.MdiParent = this;
            gridForm.StartPosition = FormStartPosition.Manual;
            gridForm.Show(); // the grid form is the initial form so show it and set current form appropriately
            currentForm = gridForm;

            statForm = new StatisticsForm(this);
            statForm.MdiParent = this;
            statForm.StartPosition = FormStartPosition.Manual;
        }

        // transition methods
        /// <summary>
        /// hides the current form, shows the statistics form and sets the current form to the statistics form
        /// </summary>
        public void showStatsScreen()
        {
            currentForm.Hide();
            statForm.UpdateData();
            statForm.Show();
            statForm.ATPButton_Click(null, null);
            currentForm = statForm;
        }
        /// <summary>
        /// hides current form, shows the grid form and sets the current form to the grid form
        /// </summary>
        public void showGridScreen()
        {
            currentForm.Hide();
            gridForm.Show();
            currentForm = gridForm;
        }

        /// <summary>
        /// Creates a new world form for the user to use and sets the form to the current form. The new world form disposes itself once the user finishes using it
        /// </summary>
        public void showNewWorldScreen()
        {
            // initialize form
            newWorldForm = new NewWorldForm(this);
            newWorldForm.MdiParent = this;
            newWorldForm.StartPosition = FormStartPosition.Manual;
            // show the form and update the current form value
            currentForm.Hide();
            newWorldForm.Show();
            currentForm = newWorldForm;
        }

        /// <summary>
        /// call the grid forms newWorld method - this is the function called by the new world form to pass the data to the grid form from the new world form
        /// </summary>
        /// <param name="length">new world length</param> 
        /// <param name="height">new world height</param> 
        /// <param name="bands">new worlds number of inital bands</param> 
        /// <param name="startingMembers">new worlds starting number of members in each band</param> 
        public void newWorld(int length, int height, int bands, int startingMembers)
        {
            gridForm.newWorld(length, height, bands, startingMembers); // call the method in the grid form
        }

        /// <summary>
        /// This method allows the statistics form to request data from the grid form, the statistic form calls this method and passes references to the lists where the data must be placed
        /// </summary>
        /// <param name="timestep">A reference to be updated with the current timestep</param>
        /// <param name="ATPValues">A reference to be updated with the Average total Prey</param>
        /// <param name="ATBValues">A reference to be updated with the Average total Barley</param>
        /// <param name="ATFValues">A reference to be updated with the Average total farm</param>
        /// <param name="TPValues">A reference to be updated with the total population values</param>
        /// <param name="TBValues">A reference to be updated with the total births</param>
        /// <param name="TDValues">A reference to be updated with the total deaths</param>
        /// <param name="BBTValues">A reference to be updated with the births by turn</param>
        /// <param name="DBTValues">A reference to be updated with the deaths by turn</param>
        public void worldData(ref int timestep,ref List<float> ATPValues, ref List<float> ATBValues, ref List<float> ATFValues, ref List<float> TPValues, ref List<float> TBValues, ref List<float> TDValues, ref List<float> BBTValues, ref List<float> DBTValues)
        {
            gridForm.worldData(ref timestep, ref ATPValues, ref ATBValues, ref ATFValues, ref TPValues, ref TBValues, ref TDValues, ref BBTValues, ref DBTValues); // call the method in the grid form
        }

        private void FormContainer_Load(object sender, EventArgs e)
        {

        }
    }
}
