﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using CapstoneProject.EngineClasses;

namespace CapstoneProject.GUI
{
    class Grid
    {
        Button[,] theGrid;

        private Form hostForm;
        private int height;      
        private int width;    
        private int xOffset;
        private int yOffset;
        private int rows;
        private int cols;

        Font defaultFont = new Font("Microsoft Sans Serif", 8.25f);
        Font smallFont = new Font("Microsoft Sans Serif", 6f);
        Font currentFont;

        public Grid(System.Windows.Forms.Form host, int heightVal, int widthVal, int xOffsetVal, int yOffsetValue, int nrRows, int nrCol)
        {
            hostForm = host;
            height = heightVal;
            xOffset = xOffsetVal;
            yOffset = yOffsetValue;
            width = widthVal;
            rows = nrRows;
            cols = nrCol;
            currentFont = defaultFont;
            
            theGrid = new Button[rows, cols];

            generateGrid();
        }

        private void generateGrid()
        {
            for (int i = 0; i < rows; i++)
            {
                for (int k = 0; k < cols; k++)
                {
                    Button aBlock = new Button();

                    aBlock.Height = height;
                    aBlock.Width = width;

                    aBlock.Top = yOffset + k * aBlock.Height;
                    aBlock.Left = xOffset + i * aBlock.Width;

                    aBlock.BackColor = Color.White;

                    aBlock.Font = currentFont;

                    //*****You have to add the control (a button in this case)
                    //***** to the Control list of the form
                    theGrid[i, k] = aBlock;
                    hostForm.Controls.Add(aBlock);
                }
            }

        }

        public void newWorldGrid(int length, int height)
        {
            foreach (Button button in theGrid)
            {
                button.Dispose();
            }
            rows = length; // the grids x and y are a swapped version of the worlds x and y
            cols = height;

            if (cols > 14 || rows > 10)
            {
                this.width = 54;
                this.height = 54;
                currentFont = smallFont;
            }
            else
            {
                this.width = 64;
                this.height = 64;
                currentFont = defaultFont;
            }

            theGrid = new Button[rows, cols];
            generateGrid();
        }

        public void updateGrid(Cell[,] cellGrid, Boolean showingNAR)
        {
            foreach(Cell cell in cellGrid)
            {
                theGrid[cell.X, cell.Y].Text = "";
                if (cell.Occupied == true)
                {
                    theGrid[cell.X, cell.Y].BackgroundImage = CapstoneProject.Properties.Resources.Band;
                    theGrid[cell.X, cell.Y].Text += cell.CellBand.Members + "\n";
                }
                else
                {
                    theGrid[cell.X, cell.Y].BackgroundImage = null;
                }
                if (!showingNAR)
                    theGrid[cell.X, cell.Y].Text += cell.CellPrey.CurrentValue + "\n" + cell.CellBarley.CurrentValue + "\n" + cell.CellFarm.CurrentValue; // current amount
                else
                    theGrid[cell.X, cell.Y].Text += cell.CellPrey.ResourceNAR + "\n" + cell.CellBarley.ResourceNAR + "\n" + cell.CellFarm.ResourceNAR; // NAR
                if (cell.CellHabitat.Name == "Lush")
                {
                    theGrid[cell.X, cell.Y].BackColor = Color.ForestGreen;
                }
                else if (cell.CellHabitat.Name == "Medium")
                {
                    theGrid[cell.X, cell.Y].BackColor = Color.Yellow;
                }
                else if (cell.CellHabitat.Name == "Desert")
                {
                    theGrid[cell.X, cell.Y].BackColor = Color.Red;
                }
            }
        }
    }
}
