﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace CapstoneProject.GUI
{
    public partial class NewWorldForm : Form
    {
        FormContainer container; // the container form that contains this screen

        // booleans to check that the data is being input correctly
        Boolean lengthSet = false;
        Boolean heightSet = false;
        Boolean initialBandsSet = false;
        Boolean startingMemberSet = false;

        public NewWorldForm(FormContainer container)
        {
            InitializeComponent();
            // setup the values in the starting band member combo box
            for (int i = 1; i < 40; i++)
            {
                startingMembersComboBox.Items.Add(i);
            }

            this.container = container; // set container
        }

        // action taken when cancel menustrip button is clicked -  shows the grid screen and disposes this screen
        private void cancelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            container.showGridScreen();
            this.Dispose();
        }

        // action taken when exit menustrip is clicked - exits the program
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        // action taken when cancel button is clicked -  shows the grid screen and disposes this screen
        private void cancelButton_Click(object sender, EventArgs e)
        {
            container.showGridScreen();
            this.Dispose();
        }

        // action taken when the confirm button is clicked
        private void confirmButton_Click(object sender, EventArgs e)
        {
            if (initialBandsSet == true && startingMemberSet == true) // make sure all the values are set - if the band values are set the dimension values must have been set
            {
                DialogResult result = MessageBox.Show("Creating a new world will cause all current world data (including data logs) to be lost. Proceed?", "Proceed?", MessageBoxButtons.YesNo); // warn user that current data and logs will be lost
                if (result == System.Windows.Forms.DialogResult.Yes) // if user accepts create the new world, change to the grid screen and dispose this
                {
                    container.newWorld(Convert.ToInt32(lengthComboBox.SelectedItem), Convert.ToInt32(heightComboBox.SelectedItem), Convert.ToInt32(initialBandsComboBox.SelectedItem), Convert.ToInt32(startingMembersComboBox.SelectedItem));
                    container.showGridScreen();
                    this.Dispose();
                }
                // otherwise do nothing and allow the user to cancel
            }
            else
            {
                MessageBox.Show("Not all fields have been set"); // if not all values are set inform the user
            }
        }

        // action taken when the length combobox has its index changed
        private void lengthComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            lengthSet = true;
            if (heightSet == true) // user can only select band data once the world dimensions are chosen
            {
                initialBandsComboBox.Enabled = true;
                startingMembersComboBox.Enabled = true;
                // populate the bands combobox using the worlds dimensions - can't have more bands than cells
                initialBandsComboBox.Items.Clear();
                for (int i = 1; i <= Convert.ToInt32(lengthComboBox.SelectedItem) * Convert.ToInt32(heightComboBox.SelectedItem); i++) // only allow bands up to the the maximum number of cells
                {
                    initialBandsComboBox.Items.Add(i);
                }
                // if the user goes back and changes the dimensions ensure that we don't exceed the max bands possible for new dimensions
                initialBandsComboBox.Text = "";
                initialBandsSet = false;
            } 
        }

        // action taken when the height combobox has its index changed
        private void heightComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            heightSet = true;
            if (lengthSet == true) // user can only select band data once the world dimensions are chosen
            {
                initialBandsComboBox.Enabled = true;
                startingMembersComboBox.Enabled = true;
                // populate the bands combobox using the worlds dimensions - can't have more bands than cells
                initialBandsComboBox.Items.Clear();
                for (int i = 1; i <= Convert.ToInt32(lengthComboBox.SelectedItem) * Convert.ToInt32(heightComboBox.SelectedItem); i++) // only allow bands up to the the maximum number of cells
                {
                    initialBandsComboBox.Items.Add(i);
                }
                // if the user goes back and changes the dimensions ensure that we don't exceed the max bands possible for new dimensions
                initialBandsComboBox.Text = "";
                initialBandsSet = false;
            }
        }

        // action taken when the initial bands combobox has its index changed - simply sets the boolean check to true
        private void initialBandsComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            initialBandsSet = true; // tell screen that this now has a value set
        }

        // action taken when the starting members combobox has its index changed - simply sets the boolean check to true
        private void startingMembersComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            startingMemberSet = true; // tell screen that this now has a value set
        }

        // TEXT UPDATE METHODS - These methods prevent user confusion by preventing the user from inputing text into the combo boxes
        // the boxes have their text value set to "" if the user types in it, making it appear like they cannot type within these boxes which is what we want
        #region Text Update Methods
        private void lengthComboBox_TextUpdate(object sender, EventArgs e)
        {
            lengthComboBox.Text = "";
        }

        private void heightComboBox_TextUpdate(object sender, EventArgs e)
        {
            heightComboBox.Text = "";
        }

        private void initialBandsComboBox_TextUpdate(object sender, EventArgs e)
        {
            initialBandsComboBox.Text = "";
        }

        private void startingMembersComboBox_TextUpdate(object sender, EventArgs e)
        {
            startingMembersComboBox.Text = "";
        }
        #endregion
    }
}
