﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace CapstoneProject.GUI
{
    public partial class StatisticsForm : Form
    {
        // The titles for the different graphs
        const string ATPTitle = "Average Total Prey";
        const string ATBTitle = "Average Total Barley";
        const string ATFTitle = "Average Total Farm";
        const string TPTitle = "Total Population";
        const string TBTitle = "Total Births";
        const string TDTitle = "Total Deaths";
        const string BBTTitle = "Births by Time Step";
        const string DBTTitle = "Deaths by Time Step";
        // the arrays that will contain the values for the different graphs
        List<float> ATPValues = new List<float>();
        List<float> ATBValues = new List<float>();
        List<float> ATFValues = new List<float>();
        List<float> TPValues = new List<float>();
        List<float> TBValues = new List<float>();
        List<float> TDValues = new List<float>();
        List<float> BBTValues = new List<float>();
        List<float> DBTValues = new List<float>();
        int timestep = 3;

        FormContainer container; // the container form that contains this screen


        public StatisticsForm(FormContainer container)
        {
            InitializeComponent();
            this.container = container; // set the container
        }

        private void StatisticsForm_Load(object sender, EventArgs e)
        {

        }

        // Action for when the exit menustrip button is clicked - exits the program
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        // Action for when the show grid menustrip button is clicked - asks the container form to call the appropriate transition method
        private void showGridToolStripMenuItem_Click(object sender, EventArgs e)
        {
            container.showGridScreen();
        }
        // Action for when the start new simulation menustrip button is clicked - asks the container form to call the appropriate method
        private void startNewSimulationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            container.showNewWorldScreen();
        }

        /// <summary>
        /// This is called when the form is transitioned to. All lists are cleared and then new data is placed into them by calling the appropriate method in th econtainer form
        /// </summary>
        public void UpdateData()
        {
            // clear all current data
            ATPValues.Clear();
            ATBValues.Clear();
            ATFValues.Clear();
            TPValues.Clear();
            TBValues.Clear();
            TDValues.Clear();
            // get new data from the engine, via the container form
            ((FormContainer)MdiParent).worldData(ref timestep,ref ATPValues, ref ATBValues, ref ATFValues, ref TPValues, ref TBValues, ref TDValues, ref BBTValues, ref DBTValues);
            Console.WriteLine("Data updated");
        }

        // action when the average total prey button is clicked - updates the graph
        public void ATPButton_Click(object sender, EventArgs e)
        {
            // make sure the graph format is correct
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Line;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(ATPTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for(int i = 0; i < timestep ; i ++)
            {
                collect.AddY(ATPValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisX.Title = "Time Step";
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Average Total Prey";
            chart1.DataSource = ATPValues;
            chart1.Update();
        }

        // action when the average total barley button is clicked - updates the graph
        private void ATBButton_Click(object sender, EventArgs e)
        {
            // make sure the graph format is correct
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Line;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(ATBTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for (int i = 0; i < timestep; i++)
            {
                collect.AddY(ATBValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Average Total Barley";
            chart1.Update();
        }

        // action when the average total farm button is clicked - updates the graph
        private void ATFButton_Click(object sender, EventArgs e)
        {
            // make sure the graph format is correct
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Line;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(ATFTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for (int i = 0; i < timestep; i++)
            {
                collect.AddY(ATFValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Average Total Farm";
            chart1.Update();
        }

        // action when the total population button is clicked - updates the graph
        private void TPButton_Click(object sender, EventArgs e)
        {
            // make sure the graph format is correct
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Line;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(TPTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for (int i = 0; i < timestep; i++)
            {
                collect.AddY(TPValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Total Population";
            chart1.Update();
        }

        // action when the total births button is clicked - updates the graph
        private void TBButton_Click(object sender, EventArgs e)
        {
            // make sure the graph format is correct
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Line;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(TBTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for (int i = 0; i < timestep; i++)
            {
                collect.AddY(TBValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Total Births";
            chart1.Update();
        }

        // action when the total deaths button is clicked - updates the graph
        private void TDButton_Click(object sender, EventArgs e)
        {
            // make sure the graph format is correct
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Line;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(TDTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for (int i = 0; i < timestep; i++)
            {
                collect.AddY(TDValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Total Deaths";
            chart1.Update();
        }

        // action when the births by timestep button is clicked - updates the graph
        private void birthsByTimestepbutton_Click(object sender, EventArgs e)
        {
            // make sure the graph format is correct
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Column;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(BBTTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for (int i = 0; i < timestep; i++)
            {
                collect.AddY(BBTValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Births for the Time Step";
            chart1.Update();
        }

        // action when the deaths by timestep button is clicked - updates the graph
        private void deathsByTimestepbutton_Click(object sender, EventArgs e)
        {
            // make sure the graph format is correct
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Column;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(DBTTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for (int i = 0; i < timestep; i++)
            {
                collect.AddY(DBTValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Deaths for the Time Step";
            chart1.Update();
        }

        /* MENU STRIP BUTTONS - these do exactly the same as the button actions above except they are accessed via the menu options rather than the buttons in the GUI - see above methods for 
           explanatory comments */
        #region Menu Strip Buttons
        private void averageTotalPreyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Line;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(ATPTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for (int i = 0; i < timestep; i++)
            {
                collect.AddY(ATPValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Average Total Prey";
            chart1.Update();
        }

        private void averageTotalBarleyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Line;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(ATBTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for (int i = 0; i < timestep; i++)
            {
                collect.AddY(ATBValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Average Total Barley";
            chart1.Update();
        }

        private void averageTotalFarmToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Line;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(ATFTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for (int i = 0; i < timestep; i++)
            {
                collect.AddY(ATFValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Average Total Farm";
            chart1.Update();
        }

        private void totalPopulationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Line;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(TPTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for (int i = 0; i < timestep; i++)
            {
                collect.AddY(TPValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Total Population";
            chart1.Update();
        }

        private void totalBirthsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Line;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(TBTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for (int i = 0; i < timestep; i++)
            {
                collect.AddY(TBValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Total Births";
            chart1.Update();
        }

        private void totalDeathsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Line;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(TDTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for (int i = 0; i < timestep; i++)
            {
                collect.AddY(TDValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Total Deaths";
            chart1.Update();
        }

        private void birthsByTimestepToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Column;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(BBTTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for (int i = 0; i < timestep; i++)
            {
                collect.AddY(BBTValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Births for the Time Step";
            chart1.Update();
        }

        private void deathsByTimestepToolStripMenuItem_Click(object sender, EventArgs e)
        {
            chart1.Series.ElementAt(0).ChartType = SeriesChartType.Column;
            // set the correct title
            chart1.Titles.Clear();
            chart1.Titles.Add(DBTTitle);
            // Clear the chart's current data
            chart1.Series.ElementAt(0).Points.Clear();
            DataPointCollection collect = chart1.Series.ElementAt(0).Points;
            // add the new data points
            for (int i = 0; i < timestep; i++)
            {
                collect.AddY(DBTValues[i]);
            }
            chart1.ChartAreas.ElementAt(0).AxisY.Title = "Deaths for the Time Step";
            chart1.Update();
        }
        #endregion

    }
}
